<?php

namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\NewCode;
use Teknisa\SM\Service\Sm00015004RecursoTarefaService;

class Sm00015004RecursoTarefaController extends SMCrud {
    
    protected $dataSourceName = "fct_tarefasagendcont_widget";
    protected $newCode;
    protected $recursoTarefaService;

    public function __construct($crudService, $environment, NewCode $newCode, Sm00015004RecursoTarefaService $recursoTarefaService) {
        parent::__construct($crudService, $environment);
        $this->newCode = $newCode;
        $this->recursoTarefaService = $recursoTarefaService;
    }
    
    public function save(Request $request, Response $response) {
        try {
            $nrorg = $this->environment->getNrOrg();
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row["NRTAREFASAGENDCONT"] = $this->newCode->getCodeByTableName("FCT_TAREFASAGENDCONT", $nrorg, 1, 10, true);
            }
            unset($row);
            
            parent::save($request, $response);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
    *  A função "recebeOrdenacao" recebe a próxima a ordenação da tarefa 
    *  que está sendo inserida para a recurso.
    */
    
    public function recebeOrdenacao(Request\Row $request, Response $response) {
    
        $row = $request->getRow();
        
        $nrRecursosAgendCont = $row["NRRECURSOSAGENDCONT"]; 
        
        $ordenacao = $this->recursoTarefaService->recebeOrdenacao($nrRecursosAgendCont);
        
        $dtsOrdenacao = new DataSet('ORDENACAO', $ordenacao);
        
        $response->addDataSet($dtsOrdenacao);
    }   
    
    
    /**
     *  A função "validar" verifica se já existe a ordenação da tarefa 
     *  que está sendo inserida para a recurso.
     */
     
    public function validar(Request $request, Response $response){
        try {
            $row = $request->getRow();
            $ordenacao = $row["IDORDENACAO"];
            $nrRecursosAgendCont = $row["NRRECURSOSAGENDCONT"];
            $nrorg = $this->environment->getNrOrg();

            $resultado = $this->recursoTarefaService->validar($ordenacao, $nrRecursosAgendCont, $nrorg);

            $response->addDataSet(
                new DataSet(
                    "validar", array(
                        "resultado" => $resultado
                    )
                )
            );
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}

?>