<?php

namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00001000DocumentoPropostaFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Exclui os documentos temporários a ser utilizado para gerar o relatório.
     */
    public function excluirDocumentosTemporarios(array $params) {
        $sql = "DELETE FROM FCT_TMP_DOCUMENTOORCA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna o documento da proposta para o escopo informado.
     */
    public function getDocumentoProposta(array $params) {
        /*$sql = "SELECT DISTINCT MODELO.NRSEQORCADOC,
                       MODELO.NMARQUIVO,
                       MODELO.DSCAMINHOARQ,
                       MODELO.DSTIPO,
                       MODELO.VRTAMANHO,
                       MODELO.DSCAMINHODOCGERADOPDF,
                       NVL(MODELO.NRTIPOORCAMENTO, RELAC.NRTIPOORCAMENTO) NRTIPOORCAMENTO,
                       RELAC.CDFILIAL,
                       MODELO.IDSITUACAOORCAMENTO,
                       MODELO.VRTAMANHOLOGO
                  FROM FCT_ORCAMENTODOC MODELO
                 INNER JOIN FCT_RELACESCOPOPROPOSTA RELAC
                    ON MODELO.NRSEQORCAMENTO = RELAC.NRSEQORCAMENTO
                   AND MODELO.NRORG          = RELAC.NRORG
                   AND ((MODELO.NRTIPOORCAMENTO = RELAC.NRTIPOORCAMENTO)
                         OR (MODELO.NRTIPOORCAMENTO IS NULL))
                 WHERE MODELO.NRSEQORCAMENTO    = :NRSEQORCAMENTO
                   AND ((MODELO.NRTIPOORCAMENTO = :NRTIPOORCAMENTO)
                    OR (:NRTIPOORCAMENTO       = 0))
                   AND MODELO.NRORG = :NRORG
                   AND ((MODELO.IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO)
                        OR (MODELO.IDSITUACAOORCAMENTO IS NULL))";*/
        
        $sql = "SELECT ORCADOC.NRSEQORCADOC,
                       ORCADOC.NRSEQMODDOCFCT,
                       DOCIDIOMA.NMARQUIVO,
                       DOCIDIOMA.DSCAMINHOARQ,
                       DOCIDIOMA.DSTIPO,
                       DOCIDIOMA.VRTAMANHO,
                       ORCADOC.DSCAMINHODOCGERADOPDF,
                       NVL(ORCADOC.NRTIPOORCAMENTO, RELAC.NRTIPOORCAMENTO) NRTIPOORCAMENTO,
                       ORCADOC.VRTAMANHOLOGO,
                       RELAC.CDFILIAL
                  FROM FCT_ORCAMENTODOCIDIOMA DOCIDIOMA
                 INNER JOIN FCT_ORCAMENTODOC ORCADOC
                    ON DOCIDIOMA.NRSEQORCADOC = ORCADOC.NRSEQORCADOC
                 INNER JOIN FCT_RELACESCOPOPROPOSTA RELAC
                    ON ORCADOC.NRSEQORCAMENTO  = RELAC.NRSEQORCAMENTO
                   AND ORCADOC.NRTIPOORCAMENTO = RELAC.NRTIPOORCAMENTO
                 INNER JOIN FCT_ORCAMENTODOCSITU DOCSITU
                    ON ORCADOC.NRSEQORCADOC = DOCSITU.NRSEQORCADOC
                 WHERE ORCADOC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND (ORCADOC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO OR :NRTIPOORCAMENTO = 0)
                   AND DOCIDIOMA.CDIDIOM = :CDIDIOM
                   AND DOCSITU.IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO
                 UNION ALL
                SELECT ORCADOC.NRSEQORCADOC,
                       ORCADOC.NRSEQMODDOCFCT,
                       ORCADOC.NMARQUIVO,
                       ORCADOC.DSCAMINHOARQ,
                       ORCADOC.DSTIPO,
                       ORCADOC.VRTAMANHO,
                       ORCADOC.DSCAMINHODOCGERADOPDF,
                       NVL(ORCADOC.NRTIPOORCAMENTO, RELAC.NRTIPOORCAMENTO) NRTIPOORCAMENTO,
                       ORCADOC.VRTAMANHOLOGO,
                       RELAC.CDFILIAL
                  FROM FCT_ORCAMENTODOC ORCADOC
                 INNER JOIN FCT_RELACESCOPOPROPOSTA RELAC
                    ON ORCADOC.NRSEQORCAMENTO  = RELAC.NRSEQORCAMENTO
                   AND ORCADOC.NRTIPOORCAMENTO = RELAC.NRTIPOORCAMENTO
                 INNER JOIN FCT_ORCAMENTODOCSITU DOCSITU
                    ON ORCADOC.NRSEQORCADOC = DOCSITU.NRSEQORCADOC
                 INNER JOIN IDIOMA
                    ON IDIOMA.CDIDIOM = :CDIDIOM
                 WHERE ORCADOC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND (ORCADOC.NRTIPOORCAMENTO = :NRTIPOORCAMENTO OR :NRTIPOORCAMENTO = 0)
                   AND DOCSITU.IDSITUACAOORCAMENTO = :IDSITUACAOORCAMENTO
                   AND IDIOMA.IDPADRAO = 'S'
                   AND NOT EXISTS (SELECT 1
                                     FROM FCT_ORCAMENTODOCIDIOMA
                                    WHERE NRSEQORCADOC = ORCADOC.NRSEQORCADOC
                                      AND CDIDIOM = :CDIDIOM)";
        
        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    
    /**
     * Retorna os dados principais da proposta.
     */
    public function getDadosProposta(array $params) {
        $sql = "SELECT ORC.NRPARCNEGOCIO,
                       PARC.NMPRINCIPALPARC AS NMPARCNEGOCIO,
                       PARC.NRINSCRICAOPARC,
                       CLI.NRINSESTCLIE,
                       ORC.CDFILIAL,
                       ORC.NRORCAMENTO,
                       ORC.NRVERSAOORCAMENTO,
                       TO_CHAR(ORC.DTREFORCAMENTO, 'DD/MM/YYYY') AS DTREFORCAMENTO,
                       ORC.CDMODPGTO,
                       MODPGTO.DSMODPGTO,
                       ORC.NRPARCINTERLOCUTOR,
                       ORC.NRSEQINTERLOCUTOR,
                       NVL(ORC.NMPARCINTERLOCUTOR, INTER.NMINTERLOCUTOR) AS NMPARCINTERLOCUTOR,
                       ORC.DSREFERENCIAORCA,
                       ORC.QTDIASVALIDADE,
                       TO_CHAR(ORC.DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE,
                       ORC.NRDIASCARENCIA,
                       ORC.QTDIASCONTRATO,
                       ORC.DSEMAIL,
                       ORC.NRTELEFONE,
                       ORC.DSOBSERVACAOORCA,
                       NVL(TO_CHAR(ORC.ISSUEID), TO_CHAR(ORC.NRSOLISERV) ) AS NRISSUESOL,
                       NVL2(ORC.ISSUEID, 'Issue', NVL2(ORC.NRSOLISERV, 'Solicitação', null)) AS TPISSUESOL,
                       ORC.ISSUEID AS NRISSUE,
                       ORC.NRSOLISERV,
                       ORC.VRORCAMENTO,
                       ORC.CDIDIOM,
                       IDIOMA.NMIDIOMA,
                       ORC.NRMOEDA
                  FROM FCT_ORCAMENTO ORC
                 INNER JOIN PARCNEGOCIO PARC
                    ON ORC.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                   AND ORC.NRORG         = PARC.NRORG
                  LEFT JOIN CLIENTE CLI
                    ON PARC.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                   AND PARC.NRORG         = CLI.NRORG
                  LEFT JOIN MODPGTO
                    ON ORC.CDMODPGTO = MODPGTO.CDMODPGTO
                   AND ORC.NRORG     = MODPGTO.NRORG
                  LEFT JOIN CRM_INTERLOCUTOR INTER
                    ON ORC.NRSEQINTERLOCUTOR = INTER.NRSEQINTERLOCUTOR
                   AND ORC.NRORG             = INTER.NRORG
                  LEFT JOIN IDIOMA
                    ON ORC.CDIDIOM = IDIOMA.CDIDIOM
                 WHERE ORC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND ORC.NRORG          = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna a descrição da modalidade de pagamento da proposta.
     */
    public function getModalidadeDePagamento(array $params) {
        $sql = "SELECT DSMODPGTO
                  FROM MODPGTO
                 WHERE CDMODPGTO = :CDMODPGTO
                   AND NRORG     = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o endereço de entrega da proposta.
     */
    public function getEnderecoDeEntregaProposta(array $params) {
        $sql = "SELECT DISTINCT (ENDE.DSENDERECO || ' - ' || NVL(BAIRRO.NMBAIRRO, ENDE.NMBAIRROENDERECO) || ' - ' || MUN.NMMUNICIPIO || ' - ' || ENDE.SGESTADO) AS ENDERECO,
                   ENDE.DSENDERECO,
                   MUN.NMMUNICIPIO,
                   NVL(BAIRRO.NMBAIRRO, ENDE.NMBAIRROENDERECO) AS NMBAIRROENDERECO,
                   ENDE.SGESTADO,
                   ENDE.NRCEPENDERECO
                FROM FCT_ENDERECOORCA ENDE
                LEFT JOIN MUNICIPIO MUN
                    ON ENDE.CDMUNICIPIO = MUN.CDMUNICIPIO
                   AND ENDE.CDPAIS      = MUN.CDPAIS
                   AND ENDE.SGESTADO    = MUN.SGESTADO
                LEFT JOIN BAIRRO
                    ON  ENDE.CDBAIRRO    = BAIRRO.CDBAIRRO
                    AND ENDE.CDMUNICIPIO = BAIRRO.CDMUNICIPIO
                    AND ENDE.CDPAIS      = BAIRRO.CDPAIS
                    AND ENDE.SGESTADO    = BAIRRO.SGESTADO
                WHERE ENDE.NRSEQORCAMENTO  = :NRSEQORCAMENTO
                   AND ENDE.NRORG          = :NRORG
                   AND ENDE.CDTIPOENDERECO = 'ENTREGA'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o vendedor principal da proposta.
     */
    public function getVendedorPrincipalProposta(array $params) {
        $sql = "SELECT PROPOSTA.NRPARCVENDEDOR,
                       VENDE.NMPRINCIPALPARC AS NMPARCVENDEDOR,
                       PROPOSTA.VRPEPARTICIPAVENDA,
                       PROPOSTA.NRPARCGERENTE,
                       GERENTE.NMPRINCIPALPARC AS NMPARCGERENTE
                  FROM (
                      SELECT NVL(VEN.NRPARCVENDEDOR, OPER.NRPARCNEGOCIO) AS NRPARCVENDEDOR,
                             VEN.NRPARCGERENTE,
                             NVL(VEN.VRPEPARTICIPAVENDA, 100) AS VRPEPARTICIPAVENDA,
                             ORC.NRORG
                        FROM FCT_ORCAMENTO ORC
                        LEFT JOIN FCT_VENDEDORORCA VEN
                          ON ORC.NRSEQORCAMENTO = VEN.NRSEQORCAMENTO
                         AND ORC.NRORG          = VEN.NRORG
                         AND VEN.IDVENPRIORCA   = 'S'
                       INNER JOIN OPERADOR OPER
                          ON ORC.CDOPERRESPONSAVEL = OPER.CDOPERADOR
                         AND ORC.NRORG             = OPER.NRORG
                       WHERE ORC.NRSEQORCAMENTO = :NRSEQORCAMENTO
                         AND ORC.NRORG          = :NRORG
                  ) PROPOSTA
                 INNER JOIN PARCNEGOCIO VENDE
                    ON PROPOSTA.NRPARCVENDEDOR = VENDE.NRPARCNEGOCIO
                   AND PROPOSTA.NRORG          = VENDE.NRORG
                  LEFT JOIN PARCNEGOCIO GERENTE
                    ON PROPOSTA.NRPARCGERENTE = GERENTE.NRPARCNEGOCIO
                   AND PROPOSTA.NRORG         = GERENTE.NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o código dos itens da proposta.
     */
    public function getCodigoItensProposta(array $params) {
        $sql = "SELECT DISTINCT ROWNUM AS IND,
                       ITEM.*
                  FROM (SELECT IT.NRITEMORCAMENTO, IT.CDSERVORCA
                          FROM FCT_ITEMORCAMENTO IT
                         LEFT JOIN FCT_ITNEGOCIACAOPROPOSTA ITNGP
                          ON ITNGP.NRITEMORCAMENTO = IT.NRITEMORCAMENTO
                          AND ITNGP.NRORG           = IT.NRORG
                         LEFT JOIN FCT_NEGOCIACAOPROPOSTA NEGP
                          ON ITNGP.NRNEGOCIACAOPROPOSTA = NEGP.NRNEGOCIACAOPROPOSTA
                          AND ITNGP.NRORG           = NEGP.NRORG
                         WHERE IT.NRSEQORCAMENTO      = :NRSEQORCAMENTO
                         AND (IT.NRTIPOORCAMENTO    = :NRTIPOORCAMENTO
                            OR IT.NRTIPOORCAMENTO    IS NULL)
                           AND IT.NRITEMORCAMENTOPAI IS NULL
                           AND IT.IDADICITEMORC       = 'N'
                           AND IT.NRORG               = :NRORG
                           AND IT.IDAPARECEPROPOSTA   = 'S'
                           AND NEGP.IDPREVCOBRFUT     = 'N'
                         ORDER BY IT.NRITEMORCAMENTO) ITEM";
                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o código dos itens adicionais da proposta.
     */
    public function getCodigoItensAdicionaisProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.*
                  FROM (
                       SELECT NRITEMORCAMENTO
                         FROM FCT_ITEMORCAMENTO
                        WHERE NRSEQORCAMENTO     = :NRSEQORCAMENTO
                          AND (NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                           OR NRTIPOORCAMENTO   IS NULL)
                          AND NRITEMORCAMENTOPAI = :NRITEMORCAMENTO
                          AND IDADICITEMORC      = 'S'
                          AND NRORG              = :NRORG
                          AND IDAPARECEPROPOSTA  = 'S'
                        ORDER BY NRITEMORCAMENTO
                       ) ITEM";
                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna os itens da proposta.
     */
    public function getItensProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       CODITEM,
                       NRITEMORCAMENTO,
                       NMITEMORCAMENTO,
                       IDTPFATURA,
                       QTITEMORCAMENTO,
                       VRUNITITEMORCA,
                       QTMEDIDAORCA,
                       CDTPMEDIDASERV,
                       DSTPMEDIDASERV,
                       VRACRESCIMO,
                       VRDESCITEM,
                       DECODE(VRTOTITEMSEMDESC, 0, 0, VRDESCITEM / VRTOTITEMSEMDESC * 100) AS VRPERCDESCITEM,
                       VRTOTITEMSEMDESC,
                       VRTOTITEMCOMDESC,
                       IDMULTMEDVRTOT,
                       QTMEDSERVORCA
                  FROM (
                      SELECT NVL(ITPROP.CDSERVORCA, ITPROP.CDPRODUTO) AS CODITEM,
                             ITPROP.NRITEMORCAMENTO,
                             ITPROP.NMITEMORCAMENTO,
                             NEG.IDTPFATURA,
                             NVL(ITPROP.QTITEMORCAMENTO, 0) AS QTITEMORCAMENTO,
                             NVL(ITPROP.VRUNITITEMORCA, 0) AS VRUNITITEMORCA,
                             NVL(ITPROP.QTMEDIDAORCA, 0) AS QTMEDIDAORCA,
                             ITPROP.CDTPMEDIDASERV,
                             MEDIDA.DSTPMEDIDASERV,
                             NVL((SELECT SUM(VRDESCITEM)
                                FROM FCT_ITNEGOCIACAOPROPOSTA
                               WHERE NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                                 AND NRITEMORCAMENTOPAI   = ITPROP.NRITEMORCAMENTO
                                 AND NRORG                = ITPROP.NRORG), NVL(ITNEG.VRDESCITEM, 0)) AS VRDESCITEM,
                             NVL(NEG.VRDESCITEM, 0) AS VRDESCNEG,
                             NVL(NEG.VRPERCDESCITEM, 0) AS VRPERCDESCNEG,
                             ITPROP.VRTOTALITEMORCA AS VRTOTITEMSEMDESC,
                             ITNEG.VRTOTITEMCOMDESC,
                             ITPROP.IDMULTMEDVRTOT,
                             (SELECT SUM(QTMEDIDAORCA)
                                FROM FCT_ITEMORCAMENTO
                               WHERE NRSEQORCAMENTO    = ITPROP.NRSEQORCAMENTO
                                 AND CDSERVORCA        = ITPROP.CDSERVORCA
                                 AND NRORG             = ITPROP.NRORG
                                 AND IDELEMENTO        IS NOT NULL
                                 AND IDAPARECEPROPOSTA = 'S') AS QTMEDSERVORCA,
                             (SELECT SUM((QTMEDIDAORCA * VRUNITITEMORCA) - (QTMEDIDAORCA * VRUNITITEMORCATBLPRECO))
                                FROM FCT_ITEMORCAMENTO
                               WHERE NRSEQORCAMENTO    = ITPROP.NRSEQORCAMENTO
                                 AND CDSERVORCA        = ITPROP.CDSERVORCA
                                 AND NRORG             = ITPROP.NRORG
                                 AND IDELEMENTO        IS NOT NULL
                                 AND IDAPARECEPROPOSTA = 'S') AS VRACRESCIMO
                        FROM FCT_ITNEGOCIACAOPROPOSTA ITNEG
                       INNER JOIN FCT_NEGOCIACAOPROPOSTA NEG
                          ON ITNEG.NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                         AND ITNEG.NRORG                = NEG.NRORG
                       INNER JOIN FCT_ORCAMENTO PROP
                          ON NEG.NRSEQORCAMENTO = PROP.NRSEQORCAMENTO
                         AND NEG.NRORG          = PROP.NRORG
                       INNER JOIN FCT_ITEMORCAMENTO ITPROP
                          ON ITNEG.NRITEMORCAMENTO = ITPROP.NRITEMORCAMENTO
                         AND ITNEG.NRORG           = ITPROP.NRORG
                        LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA
                          ON ITPROP.CDTPMEDIDASERV = MEDIDA.CDTPMEDIDASERV
                         AND ITPROP.NRORG          = MEDIDA.NRORG
                       WHERE PROP.NRSEQORCAMENTO      = :NRSEQORCAMENTO
                         AND PROP.NRORG               = :NRORG
                         AND (ITPROP.NRTIPOORCAMENTO  = :NRTIPOORCAMENTO
                          OR ITPROP.NRTIPOORCAMENTO   IS NULL)
                         AND ITNEG.NRITEMORCAMENTOPAI IS NULL
                         AND ITPROP.IDADICITEMORC     = 'N'
                         AND ITPROP.IDAPARECEPROPOSTA = 'S'
                         AND NEG.IDPREVCOBRFUT        = 'N'
                       ORDER BY ITPROP.NRITEMORCAMENTO
                  ) ITEM";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna os itens adicionais da proposta.
     */
    public function getItensAdicionaisProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.*
                  FROM (
                        SELECT DECODE(IT.CDSERVORCA, NULL, IT.CDPRODUTO, IT.CDSERVORCA) AS CODITEM,
                               NVL(IT.QTITEMORCAMENTO, 0) AS QTITEMORCAMENTO,
                               IT.IDFATURAORCA,
                               IT.NRITEMORCAMENTO,
                               IT.NMITEMORCAMENTO,
                               NVL(IT.QTMEDIDAORCA, 0) AS QTMEDIDAORCA,
                               TM.DSTPMEDIDASERV,
                               IT.IDUNIDTEMPOLOCACAO,
                               NVL(IT.QTTEMPOLOCACAO, 0) AS QTTEMPOLOCACAO,
                               NVL(IT.VRUNITITEMORCA, 0) AS VRUNITITEMORCA,
                               NVL(IT.VRPERCDESCORCA, 0) AS VRPERCDESCORCA,
                               NVL(IT.VRITEMSEMDESC, NVL(IT.VRTOTALITEMORCA, 0)) AS VRITEMSEMDESC,
                               NVL(IT.VRTOTALITEMORCA, 0) AS VRTOTALITEMORCA,
                               NVL((IT.VRTOTALITEMORCA), 0) TOTPERIODO,
                               MP.DSMODPGTO,
                               IT.NRTIPOHIERARQSERV,
                               IT.TXFORMULAITORCA
                          FROM FCT_ITEMORCAMENTO IT
                          LEFT JOIN FCT_TIPOMEDIDASERV TM
                            ON IT.CDTPMEDIDASERV = TM.CDTPMEDIDASERV
                           AND IT.NRORG          = TM.NRORG
                          LEFT JOIN MODPGTO MP
                            ON IT.CDMODPGTO = MP.CDMODPGTO
                           AND IT.NRORG     = MP.NRORG
                         WHERE NRSEQORCAMENTO      = :NRSEQORCAMENTO
                           AND (NRTIPOORCAMENTO    = :NRTIPOORCAMENTO
                            OR NRTIPOORCAMENTO    IS NULL)
                           AND NRITEMORCAMENTOPAI IS NULL
                           AND NRITEMORCAMENTOPAI = :NRITEMORCAMENTO
                           AND IDADICITEMORC      = 'S'
                           AND IT.NRORG            = :NRORG
                           AND IT.IDAPARECEPROPOSTA = 'S'
                           ORDER BY IT.NRITEMORCAMENTO) ITEM";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o valor total da proposta.
     */
    public function getValorTotalProposta(array $params) {
        $sql = "SELECT SUM(VRTOTALITEMORCA) AS TOTAL
                  FROM FCT_ITEMORCAMENTO
                 WHERE NRSEQORCAMENTO    = :NRSEQORCAMENTO
                   AND (NRTIPOORCAMENTO  = :NRTIPOORCAMENTO
                    OR NRTIPOORCAMENTO  IS NULL)
                   AND NRORG             = :NRORG
                   AND (IDADICITEMORC    = 'N'
                    OR IDADICCOBRITEMORC = 'S')";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Insere o documento na tabela temporária.
     */
    public function inserirDocumentoTemporario(array $params) {
        $sql = "INSERT INTO FCT_TMP_DOCUMENTOORCA (
                    NRSEQORCAMENTO, NRTIPOORCAMENTO, TXDOCUMENTO, NRORG)
                VALUES (
                    :NRSEQORCAMENTO, :NRTIPOORCAMENTO, :TXDOCUMENTO, :NRORG)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Busca os dados do comprador da proposta.
     */
    public function getDadosCompradorProposta(array $params) {
        $sql = "SELECT DISTINCT PARC.NRINSCRICAOPARC,
                       PARC.NMPRINCIPALPARC,
                       NVL(CLI.NRINSESTCLIE, PRO.NRINSESTPROS) AS CDINSCESTA
                  FROM PARCNEGOCIO PARC
                  LEFT JOIN PROSPECT PRO
                    ON PARC.NRPARCNEGOCIO = PRO.NRPARCNEGOCIO
                   AND PARC.NRORG         = PRO.NRORG
                  LEFT JOIN CLIENTE CLI
                    ON PARC.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                   AND PARC.NRORG         = CLI.NRORG
                 WHERE PARC.NRPARCNEGOCIO  = :NRPARCNEGOCIO
                   AND PARC.NRORG          = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o endereço de um determinado tipo do parceiro de negócio informado.
     */
    public function getEnderecoParceiro(array $params) {
        $sql = "SELECT DISTINCT (ENDE.DSENDERECO || DECODE(ENDE.NRIMOVELENDERECO, NULL, '', ' ' || ENDE.NRIMOVELENDERECO) || DECODE(ENDE.DSCOMPLEENDERECO, NULL, '', ', ' || ENDE.DSCOMPLEENDERECO) || ' - ' || NVL(BAIRRO.NMBAIRRO, ENDE.NMBAIRROENDERECO) || ', ' || MUN.NMMUNICIPIO || ' - ' || ENDE.SGESTADO) AS ENDERECO,
                       ENDE.DSENDERECO,
                       MUN.NMMUNICIPIO,
                       NVL(BAIRRO.NMBAIRRO, ENDE.NMBAIRROENDERECO) AS NMBAIRROENDERECO,
                       ENDE.SGESTADO,
                       EST.NMESTADO,
                       ENDE.NRCEPENDERECO
                FROM ENDERECOPARC ENDE
                LEFT JOIN ESTADO EST
                    ON ENDE.CDPAIS   = EST.CDPAIS
                   AND ENDE.SGESTADO = EST.SGESTADO
                LEFT JOIN MUNICIPIO MUN
                    ON ENDE.CDMUNICIPIO = MUN.CDMUNICIPIO
                   AND ENDE.CDPAIS      = MUN.CDPAIS
                   AND ENDE.SGESTADO    = MUN.SGESTADO
                LEFT JOIN BAIRRO
                    ON ENDE.CDBAIRRO     = BAIRRO.CDBAIRRO
                    AND ENDE.CDMUNICIPIO = BAIRRO.CDMUNICIPIO
                    AND ENDE.CDPAIS      = BAIRRO.CDPAIS
                    AND ENDE.SGESTADO    = BAIRRO.SGESTADO
                WHERE ENDE.NRPARCNEGOCIO   = :NRPARCNEGOCIO
                   AND ENDE.NRORG          = :NRORG
                   AND ENDE.CDTIPOENDERECO = :CDTIPOENDERECO
                   AND ENDE.CDPAIS IS NOT NULL";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o código dos elementos raiz da proposta.
     */
    public function getCodigoElementosRaizProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.*
                  FROM (
                    SELECT IT.NRITEMORCAMENTO,
                           IT.IDELEMENTO
                      FROM FCT_ITEMORCAMENTO IT
                        LEFT JOIN FCT_ITNEGOCIACAOPROPOSTA INEG
                        ON INEG.NRITEMORCAMENTO = IT.NRITEMORCAMENTO
                        LEFT JOIN FCT_NEGOCIACAOPROPOSTA NEG
                        ON NEG.NRNEGOCIACAOPROPOSTA = INEG.NRNEGOCIACAOPROPOSTA
                     WHERE IT.NRITEMORCAMENTOPAI = :NRITEMORCAMENTO
                       AND IT.NRORG              = :NRORG
                       AND IT.IDADICITEMORC      = 'N'
                       AND IT.IDAPARECEPROPOSTA  = 'S'
                       AND NEG.IDPREVCOBRFUT     = 'N'
                     ORDER BY IT.NRITEMORCAMENTO
                 ) ITEM";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna os elementos da proposta.
     */
    public function getElementosProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.QTMEDIDAORCA,
                       ITEM.IDELEMENTO AS IDELEMENTORAIZ,
                       ITEM.NMITEMORCAMENTO AS NMELEMENTORAIZ,
                       ITEM.CDTPMEDIDASERV,
                       MEDIDA.DSTPMEDIDASERV,
                       ITEM.CDTPMEDIDAVARIAVEL,
                       MEDVAR.DSTPMEDIDASERV AS DSTPMEDIDAVARIAVEL,
                       ROUND(RELAC.QTMEDIDAVARIAVEL, 2) AS QTMEDIDAVARIAVEL,
                       (RELAC.QTMEDIDAVARIAVEL - ITEM.QTMEDIDAORCA) AS QTFRANQUIA,
                       ROUND(ITEM.VRUNITITEMORCA, 2) AS VRUNITITEMORCA,
                       ROUND(ITNEG.VRTOTITEM, 2) AS VRTOTSERVORCASEMDESC,
                       ROUND(ITNEG.VRPERCDESCITEM, 2) AS VRPERCDESCITEM,
                       ROUND(ITNEG.VRDESCITEM, 2) AS VRDESCITEM,
                       ROUND(ITNEG.VRTOTITEMCOMDESC, 2) AS VRTOTSERVORCA,
                       ITEM.CDSERVORCA AS CDSERVORCARELAC,
                       NVL(ITEM.NMCONTRATUALITEMORCA, ITEM.NMITEMORCAMENTO) AS NMCONTRATUALITEMORCA
                  FROM FCT_ITEMORCAMENTO ITEM
                  LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA
                    ON ITEM.CDTPMEDIDASERV = MEDIDA.CDTPMEDIDASERV
                   AND ITEM.NRORG          = MEDIDA.NRORG
                 INNER JOIN FCT_ITNEGOCIACAOPROPOSTA ITNEG
                    ON ITEM.NRITEMORCAMENTO = ITNEG.NRITEMORCAMENTO
                   AND ITEM.NRORG           = ITNEG.NRORG
                  LEFT JOIN FCT_TIPOMEDIDASERV MEDVAR
                    ON ITEM.CDTPMEDIDAVARIAVEL = MEDVAR.CDTPMEDIDASERV
                   AND ITEM.NRORG              = MEDVAR.NRORG
                  LEFT JOIN FCT_RELACMEDIDAPROPOSTA RELAC
                    ON ITEM.NRSEQORCAMENTO     = RELAC.NRSEQORCAMENTO
                   AND ITEM.IDELEMENTO         = RELAC.IDELEMENTORAIZ
                   AND ITEM.CDTPMEDIDAVARIAVEL = RELAC.CDTPMEDIDAVARIAVEL
                   AND ITEM.NRORG              = RELAC.NRORG
                   AND RELAC.IDAPARECEPROPOSTA = 'S'
                 WHERE ITEM.IDELEMENTO       IS NOT NULL 
                   AND ITEM.CDSERVORCA        = :CDSERVORCA
                   AND ITEM.NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                   AND ITEM.NRORG             = :NRORG
                   AND ITEM.NRSEQORCAMENTO    = :NRSEQORCAMENTO
                   AND ITEM.IDAPARECEPROPOSTA = 'S'";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna os itens da proposta com seus parâmnetros de faturamento, filtrando o tipo de faturamento
     */
    public function getItensPropostaComParametrosFaturamento($nrseqorcamento, $nrtipoorcamento, $nrorg, $idfaturacontr) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.*
                  FROM (
                    SELECT ITPROP.NRITEMORCAMENTO,
                           ITPROP.NMITEMORCAMENTO,
                           NEG.NRNEGOCIACAOPROPOSTA,
                           DESCPER.QTMESESDESC,
                           TO_CHAR(DESCPER.DTFINALVALIDADEDESC, 'DD/MM/YYYY') AS DTFINALVALIDADEDESC,
                           NEG.VRTOTITEMCOMDESC,
                           TO_CHAR(PROP.DTACEITEORCA, 'DD/MM/YYYY') AS DTACEITEORCA
                      FROM FCT_ITEMORCAMENTO ITPROP
                     INNER JOIN FCT_ITNEGOCIACAOPROPOSTA ITNEG
                        ON ITPROP.NRITEMORCAMENTO = ITNEG.NRITEMORCAMENTO
                       AND ITPROP.NRORG           = ITNEG.NRORG
                     INNER JOIN FCT_NEGOCIACAOPROPOSTA NEG
                        ON ITNEG.NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                       AND ITNEG.NRORG                = NEG.NRORG
                      LEFT JOIN FCT_DESCPERIODONEGOPROP DESCPER
                        ON NEG.NRNEGOCIACAOPROPOSTA = DESCPER.NRNEGOCIACAOPROPOSTA
                       AND NEG.NRORG                = DESCPER.NRORG
                       AND DESCPER.VRPERCDESCITEM   = 100
                     INNER JOIN FCT_ORCAMENTO PROP
                        ON ITPROP.NRSEQORCAMENTO = PROP.NRSEQORCAMENTO
                       AND ITPROP.NRORG          = PROP.NRORG
                     WHERE ITPROP.NRSEQORCAMENTO     = :NRSEQORCAMENTO
                       AND ITPROP.NRTIPOORCAMENTO    = :NRTIPOORCAMENTO
                       AND ITPROP.NRORG              = :NRORG
                       AND ITPROP.NRITEMORCAMENTOPAI IS NULL
                       AND ITPROP.IDADICITEMORC      = 'N'
                       AND NEG.IDTPFATURA            IN (".$idfaturacontr.")
                     ORDER BY ITPROP.NRITEMORCAMENTO
                  ) ITEM";
        
        $params = array(
            "NRSEQORCAMENTO"  => $nrseqorcamento,
            "NRTIPOORCAMENTO" => $nrtipoorcamento,
            "NRORG"           => $nrorg
        );
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o código dos itens da proposta com tipo de faturamento igual a "Periódico" ou "Apuração"
     */
    public function getCodigoItensPrevisaoPagamentoPeriodicoOuApuracao(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.*
                  FROM (SELECT NRITEMORCAMENTO
                          FROM FCT_ITEMORCAMENTO
                         WHERE NRSEQORCAMENTO      = :NRSEQORCAMENTO
                           AND (NRTIPOORCAMENTO    = :NRTIPOORCAMENTO
                            OR NRTIPOORCAMENTO    IS NULL)
                           AND NRITEMORCAMENTOPAI IS NULL
                           AND IDADICITEMORC       = 'N'
                           AND NRORG               = :NRORG
                           AND IDFATURAORCA       IN ('A', 'X')
                         ORDER BY NRITEMORCAMENTO) ITEM";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna os itens da proposta com tipo de faturamento igual a "Não Periódico"
     */
    public function getItensPrevisaoPagamentoNaoPeriodico(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.*
                  FROM (
                      SELECT ITPROP.NRITEMORCAMENTO,
                             ITPROP.NMITEMORCAMENTO,
                             ITNEG.NRNEGOCIACAOPROPOSTA,
                             ITNEG.NRITNEGOCIACAOPROPOSTA,
                             NEG.VRTOTITEMCOMDESC,
                             TO_CHAR(PROP.DTACEITEORCA, 'DD/MM/YYYY') AS DTACEITEORCA
                        FROM FCT_ITEMORCAMENTO ITPROP
                       INNER JOIN FCT_ITNEGOCIACAOPROPOSTA ITNEG
                          ON ITPROP.NRITEMORCAMENTO = ITNEG.NRITEMORCAMENTO
                         AND ITPROP.NRORG           = ITNEG.NRORG
                       INNER JOIN FCT_NEGOCIACAOPROPOSTA NEG
                          ON ITNEG.NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                         AND ITNEG.NRORG                = NEG.NRORG
                       INNER JOIN FCT_ORCAMENTO PROP
                          ON ITPROP.NRSEQORCAMENTO = PROP.NRSEQORCAMENTO
                         AND ITPROP.NRORG          = PROP.NRORG
                       WHERE ITPROP.NRSEQORCAMENTO     = :NRSEQORCAMENTO
                         AND ITPROP.NRORG              = :NRORG
                         AND ITPROP.NRTIPOORCAMENTO    = :NRTIPOORCAMENTO
                         AND ITPROP.IDADICITEMORC      = 'N'
                         AND ITPROP.NRITEMORCAMENTOPAI IS NULL
                         AND NEG.IDTPFATURA            = 'P'
                  ) ITEM";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o prazo em dias e o percentual de cada parcela da forma de pagamento variável
     */
    public function getParcelasFormaPagamentoVariavel(array $params) {
        $sql = "SELECT NVL(NRDIASMODPGTO, 0) AS NRDIASPGTO,
                       NVL(VRPERCMODPGTO, 100) VRPERCPGTO
                  FROM PRAZOMODPGTO
                 WHERE CDMODPGTO = :CDMODPGTO
                   AND NRORG     = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Chama a procedure FCT_CALCULA_DIAPGTO
     */
    public function calcularDiaPgto(array $params) {
        $sql = "SELECT TO_CHAR(FCT_CALCULA_DIAPGTO (
                    :DTVENCIMENTO,
                    :IDTPPERIODO,
                    :CDFORMPGTO
                ), 'DD/MM/YYYY') AS FCT_CALCULA_DIAPGTO
                FROM DUAL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getUrlFileServerLogotipoParceiro(array $params) {
        $sql = "SELECT DSDIRLOGOMARCA
                  FROM PARCNEGOCIO
                 WHERE NRPARCNEGOCIO = :NRPARCNEGOCIO
                   AND NRORG         = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getNomeEscopo(array $params) {
        $sql = "SELECT NMTIPOORCAMENTO
                  FROM FCT_TIPOORCAMENTO
                 WHERE NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND NRORG           = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o código das formações de preço da proposta 
    **/
    public function getCodigoFormacoesPrecoProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       FORMACAO.*
                FROM (SELECT DISTINCT NRFORMPRECOSERVICO,
                                      NRSEQORCAMENTO,
                                      NRITEMORCAMENTO
                      FROM FCT_RELACFORMACAOPROP
                      WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                        AND NRORG          = :NRORG
                      ORDER BY NRFORMPRECOSERVICO) FORMACAO";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getCodigoItensFormacaoPrecoProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.*
                  FROM (SELECT CALC.NRCALCFORMPRECSERV
                          FROM FCT_CALCFORMPRECSERV CALC
                         WHERE CALC.NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                           AND CALC.NRORG              = :NRORG
                         START WITH CALC.NRCALCFORMPRECSERV IN (SELECT NRCALCFORMPRECSERV
                                                                  FROM FCT_CALCFORMPRECSERV
                                                                 WHERE NRFORMPRECOSERVICO     = :NRFORMPRECOSERVICO
                                                                   AND NRORG                  = :NRORG
                                                                   AND NRCALCFORMPRECSERVPAI IS NULL)
                       CONNECT BY PRIOR CALC.NRCALCFORMPRECSERV = CALC.NRCALCFORMPRECSERVPAI) ITEM";
                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getFormacoesPrecoProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       FORMACAO.*
                  FROM (SELECT FO.NRFORMPRECOSERVICO,
                               FO.NMFORMPRECOSERVICO,
                               FO.DSSINDICATO,
                               ITEM.CDSERVORCA,
                               NMITEM.NMSERVORCA
                          FROM FCT_FORMPRECOSERVICO FO
                         INNER JOIN FCT_RELACFORMACAOPROP REL
                            ON FO.NRFORMPRECOSERVICO = REL.NRFORMPRECOSERVICO
                           AND FO.NRORG              = REL.NRORG
                         LEFT JOIN FCT_ITEMORCAMENTO ITEM
                           ON ITEM.NRSEQORCAMENTO     = REL.NRSEQORCAMENTO
                          AND ITEM.NRITEMORCAMENTO    = REL.NRITEMORCAMENTO
                          AND ITEM.NRORG              = REL.NRORG 
                         LEFT JOIN FCT_SERVICOORCA NMITEM
                           ON NMITEM.CDSERVORCA = ITEM.CDSERVORCA
                          AND NMITEM.NRORG      = ITEM.NRORG 
                         WHERE REL.NRSEQORCAMENTO = :NRSEQORCAMENTO
                           AND REL.NRORG          = :NRORG
                         ORDER BY FO.NRFORMPRECOSERVICO) FORMACAO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getItensFormacaoPrecoProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       FORMACAO.*
                 FROM (SELECT DISTINCT LEVEL,
                              CALC.NRCALCFORMPRECSERV,
                              CALC.IDITEMCALCULO,
                              CALC.IDELEMENTO,
                              ELE.NMSERVORCA AS NMELEMENTO,
                              CALC.NRCALCFORMPRECSERVPAI,
                              CALC.VRPERCCALCULO,
                              CALC.VRUNITARIO,
                              CALC.QTCALCULO,
                              CALC.IDUSAMEDIDA,
                              CALC.QTMEDIDACALC,
                              CALC.VRTOTAL,
                              CALC.CDTPMEDIDASERV,
                              MED.DSTPMEDIDASERV,
                              (SELECT COUNT(*)
                                FROM FCT_CALCFORMPRECSERV
                                WHERE NRFORMPRECOSERVICO    = CALC.NRFORMPRECOSERVICO
                                  AND NRORG                 = CALC.NRORG
                                  AND NRCALCFORMPRECSERVPAI = CALC.NRCALCFORMPRECSERV) AS QTFILHOS
                         FROM FCT_CALCFORMPRECSERV CALC
                        INNER JOIN FCT_SERVICOORCA ELE
                          ON CALC.IDELEMENTO = ELE.IDELEMENTO
                          AND CALC.NRORG     = ELE.NRORG
                        LEFT JOIN FCT_TIPOMEDIDASERV MED
                            ON MED.CDTPMEDIDASERV  = CALC.CDTPMEDIDASERV
                            AND MED.NRORG          = CALC.NRORG 
                        WHERE CALC.NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                          AND CALC.NRORG              = :NRORG
                        START WITH CALC.NRCALCFORMPRECSERV IN (SELECT NRCALCFORMPRECSERV
                                                                 FROM FCT_CALCFORMPRECSERV
                                                                WHERE NRFORMPRECOSERVICO     = :NRFORMPRECOSERVICO
                                                                  AND NRORG                  = :NRORG
                                                                  AND NRCALCFORMPRECSERVPAI IS NULL)
                       CONNECT BY PRIOR CALC.NRCALCFORMPRECSERV = CALC.NRCALCFORMPRECSERVPAI) FORMACAO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna o elemento pai da composição de serviço adquirida pelo cliente
    **/
    public function getElementoPaiComposicaoServProposta(array $params){
        $sqlElePai = "SELECT DISTINCT ROWNUM AS INDELPAICS, 
                             TO_CHAR(NRCOMPSERVICO) AS NRCOMPSERVICO,
                             NMCOMPSERVICO
                        FROM FCT_COMPSERVORCAMENTO
                       WHERE NRSEQORCAMENTO   = :NRSEQORCAMENTO
                         AND NRORG            = :NRORG
                         AND IDESCOLHIDO      = 'S'
                         AND NRCOMPSERVICOPAI IS NULL";
        
        $std = $this->connection->prepare($sqlElePai);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /* Busca a árvore de elementos da composição de serviço */
    public function getElementoComposicaoServProposta(array $params){
        $sqlcompserv = "SELECT ROWNUM AS INDELCS,
                          IDARVORE,
                          LPAD(' ', (LEVEL-1) * 4, ' ') || NMELEMENTO AS NMELEMENTO,
                          IDARVOREPAI,
                          TIPO,
                          IDORDENAELEMENTO,
                          LEVEL,
                          VRTOTSERVORCA,
                          VRTOTSAJUSTE,
                          IDSERVOBRIGCOMP
                        FROM (SELECT TO_CHAR(NRCOMPSERVICO) AS IDARVORE,
                                      NMCOMPSERVICO AS NMELEMENTO,
                                      TO_CHAR(NRCOMPSERVICOPAI) AS IDARVOREPAI,
                                      'C' AS TIPO,
                                      -1 AS IDORDENAELEMENTO,
                                      NULL AS VRTOTSERVORCA,
                                      NULL AS VRTOTSAJUSTE,
                                      IDCOMPOBRIGATORIA AS IDSERVOBRIGCOMP,
                                      COMPS.NRORG,
                                      COMPS.NRSEQORCAMENTO
                                FROM FCT_COMPSERVORCAMENTO COMPS
                                WHERE COMPS.NRSEQORCAMENTO = :NRSEQORCAMENTO
                                AND COMPS.NRORG = :NRORG
                                AND COMPS.IDESCOLHIDO = 'S'
                                UNION ALL
                                SELECT DISTINCT TRAB.IDELEMENTO AS IDARVORE,
                                      ELE.NMSERVORCA AS NMELEMENTO,
                                      TO_CHAR(TRAB.NRCOMPSERVICO) AS IDARVOREPAI,
                                      'E' AS TIPO,
                                      ELE.IDORDENAELEMENTO,
                                      SUM(VRTOTSERVORCA) AS VRTOTSERVORCA,
                                      SUM(VRTOTSAJUSTE)  AS VRTOTSAJUSTE,
                                      TRAB.IDSERVOBRIGCOMP,
                                      TRAB.NRORG,
                                      TRAB.NRSEQORCAMENTO
                                FROM FCT_TRABSERVORCAMENTO TRAB
                                INNER JOIN FCT_SERVICOORCA ELE
                                ON TRAB.IDELEMENTO = ELE.IDELEMENTO
                                WHERE TRAB.NRSEQORCAMENTO = :NRSEQORCAMENTO
                                AND TRAB.NRORG = :NRORG
                                AND TRAB.IDESCOLHIDO = 'S'
                                AND TRAB.IDAPARECEPROPOSTA = 'S'
                                AND (SELECT COUNT(*)
                                      FROM FCT_TRABSERVORCAMENTO
                                      WHERE NRSEQORCAMENTO = TRAB.NRSEQORCAMENTO
                                      AND IDESCOLHIDO = 'S'
                                      AND IDELEMENTOPAI = TRAB.IDELEMENTO) = 0
                                GROUP BY TRAB.IDELEMENTO,
                                      ELE.NMSERVORCA,
                                      TO_CHAR(TRAB.NRCOMPSERVICO),
                                      'E',
                                      TRAB.IDSERVOBRIGCOMP,
                                      ELE.IDORDENAELEMENTO,
                                      TRAB.NRORG,
                                      TRAB.NRSEQORCAMENTO) ARVORE
                        WHERE (TIPO = :TIPO OR :TIPO IS NULL)
                        CONNECT BY PRIOR IDARVORE = IDARVOREPAI
                        START WITH IDARVORE  = :NRCOMPSERVICO
                        ORDER SIBLINGS BY IDORDENAELEMENTO";                      
        $std = $this->connection->prepare($sqlcompserv);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Retorna as informações do itens das formações de preço alteradas
     * durante a inserção da proposta
    **/
    public function getItensFormacaoAuxiliarProposta(array $params){
        $sql = "SELECT ROWNUM AS IND,
                       FORMACAO.*
                 FROM (SELECT DISTINCT LEVEL,
                              CALC.NRCALCFORMPRECSERV,
                              CALC.IDITEMCALCULO,
                              CALC.IDELEMENTO,
                              ELE.NMSERVORCA AS NMELEMENTO,
                              CALC.NRCALCFORMPRECSERVPAI,
                              CALC.VRPERCCALCULO,
                              CALC.VRUNITARIO,
                              CALC.QTCALCULO,
                              CALC.IDUSAMEDIDA,
                              CALC.QTMEDIDACALC,
                              CALC.VRTOTAL,
                              CALC.CDTPMEDIDASERV,
                              MED.DSTPMEDIDASERV,
                              (SELECT COUNT(*)
                                FROM FCT_CALCFORMPRECSERV_ITPRO
                                WHERE NRFORMPRECOSERVICO    = CALC.NRFORMPRECOSERVICO
                                  AND NRORG                 = CALC.NRORG
                                  AND NRCALCFORMPRECSERVPAI = CALC.NRCALCFORMPRECSERV) AS QTFILHOS
                         FROM FCT_CALCFORMPRECSERV_ITPRO CALC
                        INNER JOIN FCT_SERVICOORCA ELE
                          ON CALC.IDELEMENTO = ELE.IDELEMENTO
                          AND CALC.NRORG     = ELE.NRORG
                        LEFT JOIN FCT_TIPOMEDIDASERV MED
                            ON MED.CDTPMEDIDASERV  = CALC.CDTPMEDIDASERV
                            AND MED.NRORG          = CALC.NRORG 
                        WHERE CALC.NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO
                          AND CALC.NRORG              = :NRORG
                          AND CALC.NRSEQORCAMENTO     = :NRSEQORCAMENTO
                          AND CALC.NRITEMORCAMENTO    = :NRITEMORCAMENTO
                        START WITH CALC.NRCALCFORMPRECSERV IN (SELECT NRCALCFORMPRECSERV
                                                                 FROM FCT_CALCFORMPRECSERV_ITPRO
                                                                WHERE NRFORMPRECOSERVICO      = :NRFORMPRECOSERVICO
                                                                  AND NRSEQORCAMENTO          = :NRSEQORCAMENTO
                                                                  AND NRITEMORCAMENTO         = :NRITEMORCAMENTO
                                                                  AND NRORG                   = :NRORG
                                                                    AND NRCALCFORMPRECSERVPAI IS NULL)
                       CONNECT BY PRIOR CALC.NRCALCFORMPRECSERV = CALC.NRCALCFORMPRECSERVPAI) FORMACAO"; 
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function validaExtensaoDocx(array $params){
      $sql = "SELECT IDPERMITEDOCX
              FROM FCT_ORCAMENTO ORC
              INNER JOIN FCT_RELACESCOPOMODELO MODELO
                ON ORC.NRORCAMENTOMODELO  = MODELO.NRORCAMENTOMODELO
                AND ORC.NRORG             = MODELO.NRORG
              LEFT JOIN FCT_ORCAESCOPODOC ORCDOC
                ON MODELO.NRRELACESCOPOMODELO = ORCDOC.NRRELACESCOPOMODELO
                AND MODELO.NRORG              = ORCDOC.NRORG
              WHERE NRSEQORCAMENTO         = :NRSEQORCAMENTO
                AND MODELO.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                AND ORC.NRORG              = :NRORG
                AND (ORCDOC.IDSITUACAOORCAMENTO = ORC.IDSITUACAOORCAMENTO
                  OR ORCDOC.IDSITUACAOORCAMENTO  IS NULL)
                AND ORC.NRORG      = :NRORG
                AND ROWNUM = 1";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * 
    **/
    public function getItensPropostaComVariacaoPreco(array $params){
        $sql = "SELECT ROWNUM AS IND,
                       VARIACAO.*
                  FROM (
                     SELECT DISTINCT ITEMPAI.NRITEMORCAMENTO,
                            ITEMPAI.NMITEMORCAMENTO,
                            ITEMPAI.CDSERVORCA
                       FROM FCT_VARIACAOPRECBASESERV VAR
                      INNER JOIN FCT_ITPRECBASESERV ITVAR
                         ON VAR.NRSEQITPRECBSERV = ITVAR.NRSEQITPRECBSERV
                        AND VAR.CDTPMEDIDASERV   = ITVAR.CDTPMEDIDASERV
                        AND VAR.NRORG            = ITVAR.NRORG
                      INNER JOIN FCT_ITEMORCAMENTO ITEM
                         ON ITVAR.CDSERVORCA       = ITEM.CDSERVORCA
                        AND ITVAR.NRORG            = ITEM.NRORG
                        AND VAR.CDTPMEDIDAVARIAVEL = ITEM.CDTPMEDIDAVARIAVEL
                        AND ITVAR.CDTPMEDIDASERV   = ITEM.CDTPMEDIDASERV
                      INNER JOIN FCT_ITEMORCAMENTO ITEMPAI
                         ON ITEM.NRITEMORCAMENTOPAI = ITEMPAI.NRITEMORCAMENTO
                        AND ITEM.NRSEQORCAMENTO     = ITEMPAI.NRSEQORCAMENTO
                        AND ITEM.NRORG              = ITEMPAI.NRORG
                      INNER JOIN FCT_TIPOORCAMENTO TIPO
                         ON ITEM.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                        AND ITEM.NRORG           = TIPO.NRORG
                      WHERE ITEM.NRSEQORCAMENTO    = :NRSEQORCAMENTO
                        AND ITEM.NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                        AND ITEM.NRORG             = :NRORG
                        AND ITEM.IDELEMENTO        IS NOT NULL
                        AND ITEM.IDAPARECEPROPOSTA = 'S'
                        AND :IDSITUACAOORCAMENTO IN ('ELABORACAO', 'CANCELADO', 'ENVIADO')
                    UNION ALL
                    SELECT DISTINCT ITEMPAI.NRITEMORCAMENTO,
                           ITEMPAI.NMITEMORCAMENTO,
                           ITEMPAI.CDSERVORCA
                       FROM FCT_VARPRECBASESERVORCA VAR
                      INNER JOIN FCT_ITEMORCAMENTO ITEM
                         ON VAR.NRSEQORCAMENTO     = ITEM.NRSEQORCAMENTO
                        AND VAR.CDSERVORCA         = ITEM.CDSERVORCA
                        AND VAR.CDTPMEDIDAVARIAVEL = ITEM.CDTPMEDIDAVARIAVEL
                        AND VAR.CDTPMEDIDASERV     = ITEM.CDTPMEDIDASERV
                        AND VAR.NRORG              = ITEM.NRORG
                      INNER JOIN FCT_ITEMORCAMENTO ITEMPAI
                         ON ITEM.NRITEMORCAMENTOPAI = ITEMPAI.NRITEMORCAMENTO
                        AND ITEM.NRSEQORCAMENTO     = ITEMPAI.NRSEQORCAMENTO
                        AND ITEM.NRORG              = ITEMPAI.NRORG
                      WHERE ITEM.IDELEMENTO        IS NOT NULL
                        AND ITEM.NRSEQORCAMENTO    = :NRSEQORCAMENTO
                        AND ITEM.NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                        AND ITEM.NRORG             = :NRORG
                        AND ITEM.IDAPARECEPROPOSTA = 'S'
                        AND :IDSITUACAOORCAMENTO IN ('CONCLUIDO', 'ACEITO')
                    ) VARIACAO";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * Busca os dados da variação de preço
    **/
    public function getVariacoesPrecoProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       VARIACAO.*
                  FROM (
                     SELECT VAR.CDTPMEDIDASERV,
                            VAR.CDTPMEDIDAVARIAVEL,
                            MEDVAR.DSTPMEDIDASERV AS NMTPMEDIDAVARIAVEL,
                            VAR.QTMEDIDAINIVARIACAO,
                            VAR.QTMEDIDAFIMVARIACAO,
                            VAR.VRPRECOVARIACAO,
                            VAR.QTFRANQUIA,
                            VAR.IDMULTIPVARIACAO
                       FROM FCT_VARIACAOPRECBASESERV VAR
                      INNER JOIN FCT_ITPRECBASESERV ITVAR
                         ON VAR.NRSEQITPRECBSERV = ITVAR.NRSEQITPRECBSERV
                        AND VAR.CDTPMEDIDASERV   = ITVAR.CDTPMEDIDASERV
                        AND VAR.NRORG            = ITVAR.NRORG
                      INNER JOIN FCT_ITEMORCAMENTO ITEM
                         ON ITVAR.CDSERVORCA       = ITEM.CDSERVORCA
                        AND ITVAR.NRORG            = ITEM.NRORG
                        AND ITVAR.CDTPMEDIDASERV   = ITEM.CDTPMEDIDASERV
                        AND VAR.CDTPMEDIDAVARIAVEL = ITEM.CDTPMEDIDAVARIAVEL
                      INNER JOIN FCT_ITEMORCAMENTO ITEMPAI
                         ON ITEM.NRITEMORCAMENTOPAI = ITEMPAI.NRITEMORCAMENTO
                        AND ITEM.NRSEQORCAMENTO     = ITEMPAI.NRSEQORCAMENTO
                        AND ITEM.NRORG              = ITEMPAI.NRORG
                      INNER JOIN FCT_TIPOORCAMENTO TIPO
                         ON ITEM.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                        AND ITEM.NRORG           = TIPO.NRORG
                      INNER JOIN FCT_TIPOMEDIDASERV MEDVAR
                         ON VAR.CDTPMEDIDAVARIAVEL = MEDVAR.CDTPMEDIDASERV
                        AND VAR.NRORG              = MEDVAR.NRORG
                      WHERE ITEM.NRSEQORCAMENTO    = :NRSEQORCAMENTO
                        AND ITEM.NRTIPOORCAMENTO   = :NRTIPOORCAMENTO
                        AND ITEM.NRORG             = :NRORG
                        AND ITEM.IDELEMENTO        IS NOT NULL
                        AND ITEM.IDAPARECEPROPOSTA = 'S'
                        AND ITVAR.CDSERVORCA     = :CDSERVORCA
                        AND :IDSITUACAOORCAMENTO IN ('ELABORACAO', 'CANCELADO', 'ENVIADO')
                      UNION ALL
                     SELECT VAR.CDTPMEDIDASERV,
                            VAR.CDTPMEDIDAVARIAVEL,
                            MEDVAR.DSTPMEDIDASERV AS NMTPMEDIDAVARIAVEL,
                            VAR.QTMEDIDAINIVARIACAO,
                            VAR.QTMEDIDAFIMVARIACAO,
                            VAR.VRPRECOVARIACAO,
                            VAR.QTFRANQUIA,
                            VAR.IDMULTIPVARIACAO
                       FROM FCT_VARPRECBASESERVORCA VAR
                      INNER JOIN FCT_ITEMORCAMENTO ITEM
                         ON VAR.NRSEQORCAMENTO     = ITEM.NRSEQORCAMENTO
                        AND VAR.CDSERVORCA         = ITEM.CDSERVORCA
                        AND VAR.CDTPMEDIDAVARIAVEL = ITEM.CDTPMEDIDAVARIAVEL
                        AND VAR.CDTPMEDIDASERV     = ITEM.CDTPMEDIDASERV
                        AND VAR.NRORG              = ITEM.NRORG
                      INNER JOIN FCT_ITEMORCAMENTO ITEMPAI
                         ON ITEM.NRITEMORCAMENTOPAI = ITEMPAI.NRITEMORCAMENTO
                        AND ITEM.NRSEQORCAMENTO     = ITEMPAI.NRSEQORCAMENTO
                        AND ITEM.NRORG              = ITEMPAI.NRORG
                      INNER JOIN FCT_TIPOMEDIDASERV MEDVAR
                         ON VAR.CDTPMEDIDAVARIAVEL = MEDVAR.CDTPMEDIDASERV
                        AND VAR.NRORG              = MEDVAR.NRORG
                      WHERE ITEM.IDELEMENTO        IS NOT NULL
                        AND ITEM.NRSEQORCAMENTO    = :NRSEQORCAMENTO
                        AND ITEM.NRORG             = :NRORG
                        AND ITEM.CDSERVORCA        = :CDSERVORCA
                        AND ITEM.IDAPARECEPROPOSTA = 'S'
                        /*AND (CDTPMEDIDAVARIAVEL = :CDTPMEDIDAVARIAVEL
                          OR (CDTPMEDIDAVARIAVEL IS NULL AND CDTPMEDIDASERV = :CDTPMEDIDAVARIAVEL))*/
                        AND :IDSITUACAOORCAMENTO IN ('CONCLUIDO', 'ACEITO')
                    ) VARIACAO
                    ORDER BY VARIACAO.CDTPMEDIDAVARIAVEL,
                             VARIACAO.QTMEDIDAINIVARIACAO";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    /**
     * retorna a menor data de início do faturamento dos itens da proposta 
    **/
    public function getMenorDataInicioFatItens(array $params){
        $sql = "SELECT TO_CHAR(MIN(DTINICFATORC), 'DD/MM/YYYY') AS DTINICFATORC
                FROM FCT_ITEMORCAMENTO 
                WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO
                  AND NRORG          = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $dataset = $std->fetchAll();
        return !empty($dataset) ? $dataset[0]["DTINICFATORC"] : null;
    }

    public function getParcelasNegociacao(array $params) {
        $sql = "SELECT PARC.QTDIASFATURAMENTO,
                       PARC.NRDIAVENCIMENTO,
                       PARC.VRPARCELA,
                       ITNEG.VRTOTITEMCOMDESC AS VRITEM,
                       NEG.VRTOTITEMCOMDESC AS VRNEG,
                       NEG.VRPERCDESCITEM AS VRPERCDESCNEG,
                       (NEG.VRTOTITEMCOMDESC - PARC.VRPARCELA) AS DIFERENCA,
                       DECODE(NEG.VRTOTITEMCOMDESC, 0, 0, (NEG.VRTOTITEMCOMDESC - PARC.VRPARCELA) / NEG.VRTOTITEMCOMDESC * 100) AS VRPERCDESCPARC,
                       NVL(PFITESC.IDTPCALCVENC, PFESC.IDTPCALCVENC) AS IDTPCALCVENC,
                       NVL(PFITESC.NRDIACALCVENC, PFESC.NRDIACALCVENC) AS NRDIACALCVENC,
                       NVL(PFITESC.IDMESCALCVENC, PFESC.IDMESCALCVENC) AS IDMESCALCVENC
                  FROM FCT_PARCELANEGOPROP PARC
                 INNER JOIN FCT_NEGOCIACAOPROPOSTA NEG
                    ON PARC.NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                   AND PARC.NRORG                = NEG.NRORG
                 INNER JOIN FCT_ITNEGOCIACAOPROPOSTA ITNEG
                    ON PARC.NRNEGOCIACAOPROPOSTA = ITNEG.NRNEGOCIACAOPROPOSTA
                   AND PARC.NRORG                = ITNEG.NRORG
                 INNER JOIN FCT_CAPAPARAMFATESC PFESC
                    ON NEG.NRTIPOORCAMENTO = PFESC.NRTIPOORCAMENTO
                   AND NEG.NRORG           = PFESC.NRORG
                  LEFT JOIN FCT_ITESCOPOPROPOSTA ITESC
                    ON NEG.NRTIPOORCAMENTO = ITESC.NRTIPOORCAMENTO
                   AND NEG.NRORG           = ITESC.NRORG
                   AND ITNEG.CDSERVORCA    = ITESC.CDSERVORCA
                  LEFT JOIN FCT_PARAMFATITESC PFITESC
                    ON ITESC.NRITESCOPOPROPOSTA = PFITESC.NRITESCOPOPROPOSTA
                   AND ITESC.NRORG              = PFITESC.NRORG
                 WHERE PARC.NRNEGOCIACAOPROPOSTA    = :NRNEGOCIACAOPROPOSTA
                   AND PARC.NRORG                   = :NRORG
                   AND ITNEG.NRITNEGOCIACAOPROPOSTA = :NRITNEGOCIACAOPROPOSTA
                 ORDER BY PARC.QTDIASFATURAMENTO";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getPrimeiraCompetenciaNegociacaoProposta(array $params) {
        $sql = "SELECT * FROM (
                    SELECT *
                      FROM FCT_COMPETENCIANEGOPROP
                     WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                       AND NRORG                = :NRORG
                     ORDER BY IDMESCOMPETINI, NRDIACOMPETINI
                ) COMP
                WHERE ROWNUM = 1";

        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCodigoNegociacoesValores(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       NEGO.*
                  FROM (SELECT NRNEGOCIACAOPROPOSTA
                          FROM FCT_NEGOCIACAOPROPOSTA
                         WHERE NRSEQORCAMENTO  = :NRSEQORCAMENTO
                           AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                           AND NRORG           = :NRORG
                           AND IDPREVCOBRFUT   = 'N'
                         ORDER BY NRNEGOCIACAOPROPOSTA) NEGO";
                   
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getNegociacoesValores(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       NEGO.*
                  FROM (SELECT NEG.NRNEGOCIACAOPROPOSTA,
                               NEG.NRTIPOORCAMENTO,
                               NEG.IDTPFATURA,
                               NEG.CDFILIALFATURA,
                               NEG.CDTIPOOPERFATURA,
                               NEG.CDSERIENF,
                               NEG.CDEMPRESA,
                               NEG.CDINSCESTA,
                               NEG.CDINSCMUNI,
                               NEG.VRTOTITEM,
                               NEG.VRPERCDESCITEM,
                               NEG.VRDESCITEM,
                               NEG.VRTOTITEMCOMDESC,
                               NVL(FCT_BUSCA_DESCRICAO_ESCOPO(NEG.NRTIPOORCAMENTO, NEG.IDTPFATURA, NEG.CDFILIALFATURA, NEG.NRPARCFATURA, NEG.CDTIPOOPERFATURA, NEG.CDSERIENF, NEG.CDEMPRESA, NEG.CDINSCESTA, NEG.CDINSCMUNI, NEG.NRORG, NEG.IDPREVCOBRFUT), TIPO.NMTIPOORCAMENTO) AS DSNOMEAGRUPAMENTO
                          FROM FCT_NEGOCIACAOPROPOSTA NEG
                          INNER JOIN FCT_TIPOORCAMENTO TIPO
                            ON NEG.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                           AND NEG.NRORG           = TIPO.NRORG
                         WHERE NEG.NRSEQORCAMENTO  = :NRSEQORCAMENTO
                           AND NEG.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                           AND NEG.NRORG           = :NRORG
                           AND NEG.IDPREVCOBRFUT   = 'N'
                         ORDER BY NEG.NRNEGOCIACAOPROPOSTA) NEGO";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCodigoNegociacoesValoresDeParcelamento(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       NEGO.*
                  FROM (SELECT NEGO.NRNEGOCIACAOPROPOSTA,
                               NEGO.VRTOTITEMCOMDESC,
                               TO_CHAR(ORCA.DTACEITEORCA, 'DD/MM/YYYY') AS DTACEITEORCA
                          FROM FCT_NEGOCIACAOPROPOSTA NEGO
                         INNER JOIN FCT_ORCAMENTO ORCA
                            ON NEGO.NRSEQORCAMENTO = ORCA.NRSEQORCAMENTO
                           AND NEGO.NRORG          = ORCA.NRORG
                         WHERE NEGO.NRSEQORCAMENTO  = :NRSEQORCAMENTO
                           AND NEGO.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                           AND NEGO.NRORG           = :NRORG
                           AND NEGO.IDTPFATURA      = 'P'
                           AND NEGO.IDPREVCOBRFUT   = 'N'
                         ORDER BY NEGO.NRNEGOCIACAOPROPOSTA) NEGO";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getNegociacoesValoresDeParcelamento(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       NEGO.*
                  FROM (SELECT NEG.NRNEGOCIACAOPROPOSTA,
                               NEG.NRTIPOORCAMENTO,
                               NEG.VRTOTITEMCOMDESC,
                               NVL(FCT_BUSCA_DESCRICAO_ESCOPO(NEG.NRTIPOORCAMENTO, NEG.IDTPFATURA, NEG.CDFILIALFATURA, NEG.NRPARCFATURA, NEG.CDTIPOOPERFATURA, NEG.CDSERIENF, NEG.CDEMPRESA, NEG.CDINSCESTA, NEG.CDINSCMUNI, NEG.NRORG, NEG.IDPREVCOBRFUT), TIPO.NMTIPOORCAMENTO) AS DSNOMEAGRUPAMENTO
                          FROM FCT_NEGOCIACAOPROPOSTA NEG
                         INNER JOIN FCT_TIPOORCAMENTO TIPO
                            ON NEG.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                           AND NEG.NRORG           = TIPO.NRORG
                         WHERE NEG.NRSEQORCAMENTO  = :NRSEQORCAMENTO
                           AND NEG.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                           AND NEG.NRORG           = :NRORG
                           AND NEG.IDTPFATURA      = 'P'
                           AND NEG.IDPREVCOBRFUT   = 'N'
                         ORDER BY NEG.NRNEGOCIACAOPROPOSTA) NEGO";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getParcelasNegociacaoValores(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       PARC.*
                  FROM (SELECT PARC.NRPARCELANEGOPROP,
                               PARC.QTDIASFATURAMENTO,
                               PARC.NRDIAVENCIMENTO,
                               PARC.VRPARCELA,
                               PFESC.IDTPCALCVENC,
                               PFESC.NRDIACALCVENC,
                               PFESC.IDMESCALCVENC
                          FROM FCT_PARCELANEGOPROP PARC
                         INNER JOIN FCT_NEGOCIACAOPROPOSTA NEGO
                            ON PARC.NRNEGOCIACAOPROPOSTA = NEGO.NRNEGOCIACAOPROPOSTA
                           AND PARC.NRORG                = NEGO.NRORG
                         INNER JOIN FCT_CAPAPARAMFATESC PFESC
                            ON NEGO.NRTIPOORCAMENTO = PFESC.NRTIPOORCAMENTO
                           AND NEGO.NRORG           = PFESC.NRORG
                         WHERE PARC.NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                           AND PARC.NRORG                = :NRORG
                         ORDER BY PARC.QTDIASFATURAMENTO) PARC";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCodigoNegociacoesValoresParaDescontoPorPeriodo(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       NEGO.*
                  FROM (SELECT NRNEGOCIACAOPROPOSTA
                          FROM FCT_NEGOCIACAOPROPOSTA
                         WHERE NRSEQORCAMENTO  = :NRSEQORCAMENTO
                           AND NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                           AND NRORG           = :NRORG
                           AND IDPREVCOBRFUT   = 'N'
                           AND IDTPFATURA      IN ('A', 'X')
                           AND EXISTS (SELECT 1
                                         FROM FCT_DESCPERIODONEGOPROP
                                        WHERE NRNEGOCIACAOPROPOSTA = FCT_NEGOCIACAOPROPOSTA.NRNEGOCIACAOPROPOSTA
                                          AND NRORG                = FCT_NEGOCIACAOPROPOSTA.NRORG)
                         ORDER BY NRNEGOCIACAOPROPOSTA) NEGO";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getNegociacoesValoresParaDescontoPorPeriodo(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       NEGO.*
                  FROM (SELECT NEGO.NRNEGOCIACAOPROPOSTA,
                               NEGO.VRTOTITEMCOMDESC,
                               NVL(FCT_BUSCA_DESCRICAO_ESCOPO(NEGO.NRTIPOORCAMENTO, NEGO.IDTPFATURA, NEGO.CDFILIALFATURA, NEGO.NRPARCFATURA, NEGO.CDTIPOOPERFATURA, NEGO.CDSERIENF, NEGO.CDEMPRESA, NEGO.CDINSCESTA, NEGO.CDINSCMUNI, NEGO.NRORG, NEGO.IDPREVCOBRFUT), TIPO.NMTIPOORCAMENTO) AS DSNOMEAGRUPAMENTO
                          FROM FCT_NEGOCIACAOPROPOSTA NEGO
                         INNER JOIN FCT_TIPOORCAMENTO TIPO
                            ON NEGO.NRTIPOORCAMENTO = TIPO.NRTIPOORCAMENTO
                           AND NEGO.NRORG           = TIPO.NRORG
                         WHERE NEGO.NRSEQORCAMENTO  = :NRSEQORCAMENTO
                           AND NEGO.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                           AND NEGO.NRORG           = :NRORG
                           AND NEGO.IDPREVCOBRFUT   = 'N'
                           AND NEGO.IDTPFATURA      IN ('A', 'X')
                           AND EXISTS (SELECT 1
                                         FROM FCT_DESCPERIODONEGOPROP
                                        WHERE NRNEGOCIACAOPROPOSTA = NEGO.NRNEGOCIACAOPROPOSTA
                                          AND NRORG                = NEGO.NRORG)
                         ORDER BY NEGO.NRNEGOCIACAOPROPOSTA) NEGO";
                   
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getCodigoDescontosPorPeriodoNegociacaoValores(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ITEM.*
                  FROM (
                    SELECT NRDESCPERIODONEGOPROP
                      FROM FCT_DESCPERIODONEGOPROP
                     WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                       AND NRORG                = :NRORG
                     ORDER BY NRORDEMAPLICADESC
                 ) ITEM";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function getDescontosPorPeriodoNegociacaoValores(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       DESCONTO.*
                  FROM (
                    SELECT NRDESCPERIODONEGOPROP,
                           VRPERCDESCITEM,
                           VRDESCITEM,
                           VRTOTITEMCOMDESC,
                           QTMESESDESC,
                           DTFINALVALIDADEDESC,
                           NRORDEMAPLICADESC
                      FROM FCT_DESCPERIODONEGOPROP
                     WHERE NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                       AND NRORG                = :NRORG
                     ORDER BY NRORDEMAPLICADESC
                 ) DESCONTO";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }

    public function getParametrosFaturamentoItensNegociacao(array $params) {
        $sql = "SELECT PFIT.IDTPCALCVENC,
                       PFIT.QTDIASFATUVENC,
                       PFIT.NRDIACALCVENC,
                       PFIT.IDMESCALCVENC,
                       PFIT.QTDIASAPURACAO
                  FROM FCT_ITNEGOCIACAOPROPOSTA ITNEG
                 INNER JOIN FCT_NEGOCIACAOPROPOSTA NEG
                    ON ITNEG.NRNEGOCIACAOPROPOSTA = NEG.NRNEGOCIACAOPROPOSTA
                   AND ITNEG.NRORG                = NEG.NRORG
                 INNER JOIN FCT_ITESCOPOPROPOSTA ITESC
                    ON NEG.NRTIPOORCAMENTO = ITESC.NRTIPOORCAMENTO
                   AND ITNEG.CDSERVORCA    = ITESC.CDSERVORCA
                   AND ITNEG.NRORG         = ITESC.NRORG
                 INNER JOIN FCT_PARAMFATITESC PFIT
                    ON ITESC.NRITESCOPOPROPOSTA = PFIT.NRITESCOPOPROPOSTA
                   AND ITESC.NRORG              = PFIT.NRORG
                 WHERE ITNEG.NRNEGOCIACAOPROPOSTA = :NRNEGOCIACAOPROPOSTA
                   AND ITNEG.NRORG                = :NRORG
                   AND ITNEG.NRITEMORCAMENTOPAI   IS NULL";
               
       $ps = $this->connection->prepare($sql);
       $ps->execute($params);
       return $ps->fetchAll();
    }
    
    /**
     * Substitui os dados de alteração contratual
     */
    public function getDadosAlteracaoContratual(array $params) {
        $sql = "SELECT NVL(FCT_RELACESCOPOPROPOSTA.DSALTERACONTRATUAIS, FCT_ORCAMENTO.DSALTERACONTRATUAIS) AS DSALTERACONTRATUAIS
                  FROM FCT_ORCAMENTO
                  LEFT JOIN FCT_RELACESCOPOPROPOSTA
                    ON FCT_ORCAMENTO.NRSEQORCAMENTO = FCT_RELACESCOPOPROPOSTA.NRSEQORCAMENTO
                   AND FCT_ORCAMENTO.NRORG          = FCT_RELACESCOPOPROPOSTA.NRORG
                 WHERE FCT_ORCAMENTO.NRSEQORCAMENTO            = :NRSEQORCAMENTO
                   AND FCT_RELACESCOPOPROPOSTA.NRTIPOORCAMENTO = :NRTIPOORCAMENTO
                   AND FCT_ORCAMENTO.NRORG                     = :NRORG";
               
       $ps = $this->connection->prepare($sql);
       $ps->execute($params);
       return $ps->fetchAll();
    }
    
     /**
     * Retorna os dados de assinatura da proposta
     */
    public function getAssinaturasProposta(array $params) {
        $sql = "SELECT ROWNUM AS IND,
                       ASSINAT.*
                  FROM (SELECT
                            FCT_ASSINATDIGITALPROP.NRASSDIGITALORCA,
                            FCT_ASSINATDIGITALPROP.NRSEQORCAMENTO,
                            FCT_ASSINATDIGITALPROP.NRPARCNEGOCIO,
                            FCT_ASSINATDIGITALPROP.NRSEQINTERLOCUTOR,
                            DECODE(FCT_ASSINATDIGITALPROP.NRPARCNEGOCIO, NULL, CRM_INTERLOCUTOR.NMINTERLOCUTOR, PARCNEGOCIO.NMPRINCIPALPARC) AS NMSIGNATARIO,
                            FCT_ASSINATDIGITALPROP.NRCPFSIGNATARIO,
                            FCT_ASSINATDIGITALPROP.IDTIPOASSINATURA,
                            FCT_ASSINATDIGITALPROP.IDTESTEMUNHA,
                            FCT_ASSINATDIGITALPROP.NRORDENACAO,
                            FCT_ASSINATDIGITALPROP.IDSITUACAO,
                            FCT_ASSINATDIGITALPROP.IDATIVO,
                            FCT_ASSINATDIGITALPROP.NRORG
                        FROM
                            FCT_ASSINATDIGITALPROP
                            LEFT JOIN PARCNEGOCIO ON FCT_ASSINATDIGITALPROP.NRPARCNEGOCIO = PARCNEGOCIO.NRPARCNEGOCIO
                                  AND FCT_ASSINATDIGITALPROP.NRORG = PARCNEGOCIO.NRORG
                            LEFT JOIN CRM_INTERLOCUTOR ON FCT_ASSINATDIGITALPROP.NRSEQINTERLOCUTOR = CRM_INTERLOCUTOR.NRSEQINTERLOCUTOR
                                  AND FCT_ASSINATDIGITALPROP.NRORG = CRM_INTERLOCUTOR.NRORG
                            JOIN FCT_ORCAMENTO ON FCT_ASSINATDIGITALPROP.NRSEQORCAMENTO = FCT_ORCAMENTO.NRSEQORCAMENTO
                             AND FCT_ASSINATDIGITALPROP.NRORG = FCT_ORCAMENTO.NRORG
                        WHERE FCT_ASSINATDIGITALPROP.NRSEQORCAMENTO = :NRSEQORCAMENTO
                          AND FCT_ASSINATDIGITALPROP.NRORG = :NRORG
                        ORDER BY
                            FCT_ASSINATDIGITALPROP.NRORDENACAO,
                            PARCNEGOCIO.NMPRINCIPALPARC,
                            CRM_INTERLOCUTOR.NMINTERLOCUTOR) ASSINAT";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getDescricaoElementos(array $params) {
        $sql = "SELECT DISTINCT TRAB.IDELEMENTO,
                       ELEMENTO.NMSERVORCA AS NMELEMENTO,
                       ELEMENTO.DSSERVORCA AS DSELEMENTO
                  FROM FCT_TRABSERVORCAMENTO TRAB
                  JOIN FCT_SERVICOORCA ELEMENTO
                    ON TRAB.IDELEMENTO = ELEMENTO.IDELEMENTO
                 WHERE TRAB.NRSEQORCAMENTO = :NRSEQORCAMENTO
                   AND TRAB.NRORG          = :NRORG
                   AND TRAB.IDESCOLHIDO    = 'S'
                   AND ELEMENTO.DSSERVORCA IS NOT NULL";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}

?>